@@ -1,24 +1,23 @@
 package org.ddahl.rscala.server
 
-import scala.annotation.tailrec
-import scala.tools.nsc.interpreter.{IMain,ILoop}
+import scala.tools.nsc.interpreter.IMain
 import scala.tools.nsc.interpreter.IR.Success
 import scala.tools.nsc.Settings
-import java.net._
 import java.io._
+import java.nio.ByteBuffer
 
 import org.ddahl.rscala._
 import Protocol._
  
-class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, baosOut: ByteArrayOutputStream, baosErr: ByteArrayOutputStream, portsFilename: String, debugger: Debugger, serializeOutput: Boolean, rowMajor: Boolean, port: Int) {
+class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, baosOut: ByteArrayOutputStream, baosErr: ByteArrayOutputStream, portFilename: String, debugger: Debugger, serializeOutput: Boolean, rowMajor: Boolean, port: Int, bufferSize: Int) {
 
-  private val sockets = new ScalaSockets(portsFilename,port,debugger)
-  import sockets.{in, out, socketIn, socketOut}
-  import Helper.{isMatrix, transposeIf}
+  private val socket = new ScalaSocket(portFilename,port,bufferSize,debugger)
+
+  socket.putScalarInt(OK)
+  socket.flush()
+
+  private val R = RClient(this,null,socket,debugger,serializeOutput,rowMajor)
 
-  out.writeInt(OK)
-  out.flush
-  private val R = RClient(this,null,in,out,debugger,serializeOutput,rowMajor)
   if ( repl.bind("R","org.ddahl.rscala.RClient",R) != Success ) sys.error("Problem binding R.")
   if ( repl.interpret("import org.ddahl.rscala.{EphemeralReference, PersistentReference}") != Success ) sys.error("Problem interpreting import statement.")
   if ( serializeOutput ) {
@@ -32,6 +31,7 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
   private val nullary = Class.forName("scala.Function0").getMethod("apply")
   nullary.setAccessible(true)
   private val functionMap = new scala.collection.mutable.HashMap[String,(Any,String)]()
+  private val functionMap2 = new scala.collection.mutable.HashMap[String,String]()
 
   private def typeOfTerm(id: String) = repl.symbolOfLine(id).info.toString
 
@@ -41,191 +41,156 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
     mrv
   }
 
-  private def extractReturnType(signature: String) = {
-    var squareCount = 0
-    var parenCount = 0
-    var i = 0
-    val s = signature.trim
-    while ( ( ( parenCount > 0 ) || ( squareCount > 0 ) || ( ! ( ( s(i) == '=' ) && ( s(i+1) == '>' ) ) ) ) && ( i < s.length ) ) {
-      val c = s(i)
-      i += 1
-      if ( c == '(' ) parenCount += 1
-      if ( c == ')' ) parenCount -= 1
-      if ( c == '[' ) squareCount += 1
-      if ( c == ']' ) squareCount -= 1
-    }
-    if ( i == s.length ) throw new IllegalArgumentException("Unexpected end of signature."+signature)
-    i += 2
-    val r = s.substring(i).trim
-    if ( r.length == 0 ) throw new IllegalArgumentException("Unexpected end of signature."+signature)
-    r
-  }
-
-  private def readString() = Helper.readString(in)
-  private def writeString(string: String): Unit = {
-    if ( debugger.value ) debugger.msg("Writing string: <"+string+">")
-    Helper.writeString(out,string)
-  }
-
   private def setAVM(identifier: String, t: String, v: Any): Unit = {
     if ( debugger.value ) debugger.msg("Value is "+v)
     repl.bind(identifier,t,v)
   }
 
-  private val sep = sys.props("line.separator")
-
   private def doFree(): Unit = {
-    val nItems = in.readInt()
+    val nItems = socket.getScalarInt()
     if ( debugger.value ) debugger.msg("Freeing "+nItems+" cached items no longer needed by R interpreter.")
-    for ( i <- 0 until nItems ) cacheMap.free(readString())
+    for ( i <- 0 until nItems ) cacheMap.free(socket.getScalarString())
   }
 
   private def doEval(): Unit = {
-    val snippet = "var _rsMRVDummy = null\n" + readString()
+    val snippet = "var _rsMRVDummy = null\n" + socket.getScalarString()
     try {
       val result = repl.interpret(snippet)
       if ( result == Success ) {
-        if ( debugger.value ) debugger.msg("Eval is okay.")
+        if ( debugger.value ) debugger.msg("Evaluation is okay.")
         R.exit()
-        out.writeInt(OK)
+        if ( debugger.value ) debugger.msg("Exited from R.")
+        socket.putScalarInt(OK)
       } else {
         if ( debugger.value ) debugger.msg("Eval had a parse error.")
         R.exit()
-        out.writeInt(ERROR)
+        socket.putScalarInt(ERROR)
       }
     } catch {
       case e: Throwable =>
         if ( debugger.value ) debugger.msg("Caught throwable: "+e.toString)
         R.exit()
-        out.writeInt(ERROR)
+        socket.putScalarInt(ERROR)
         e.printStackTrace(pw)
         pw.println(e + ( if ( e.getCause != null ) System.lineSeparator + e.getCause else "" ) )
     }
   }
 
   private def doDef(): Unit = {
+    val body = "() => {\n" + socket.getScalarString() + "\n}"
     try {
-      val functionName = readString()
-      val functionReturnType = readString()
-      functionMap(functionName) = (repl.valueOfTerm(functionName).get, functionReturnType)
-      out.writeInt(OK)
+      if ( debugger.value ) debugger.msg("Function is: "+body)
+      val (f, returnType) = if ( ! functionMap2.contains(body) ) {
+        val result = repl.interpret(body)
+        if ( result != Success ) {
+          if ( debugger.value ) debugger.msg("Error in defining function.")
+          socket.putScalarInt(ERROR)
+          return
+        }
+        val functionName = repl.mostRecentVar
+        val f = repl.valueOfTerm(functionName).get
+        val returnType = repl.symbolOfLine(functionName).info.toString.substring(6)  // Drop "() => " in the return type.
+        functionMap2(body) = functionName
+        functionMap(functionName) = (f, returnType)
+        if ( debugger.value ) debugger.msg("Function definition is okay.")
+        (f, returnType)
+      } else functionMap(functionMap2(body))
+      socket.putScalarInt(OK)
+      socket.putScalarString(functionMap2(body))
     } catch {
       case e: Throwable =>
-        out.writeInt(ERROR)
+        socket.putScalarInt(ERROR)
         e.printStackTrace(pw)
         pw.println(e + ( if ( e.getCause != null ) System.lineSeparator + e.getCause else "" ) )
     }
   }
 
   private def doInvoke(): Unit = {
+    val functionName = socket.getScalarString()
     try {
-      val functionName = readString()
       val (f, returnType) = functionMap(functionName)
-      if ( debugger.value ) {
-        debugger.msg("Function is: "+functionName)
-        debugger.msg("... with return type: "+returnType)
-      }
       functionResult = (nullary.invoke(f), returnType)
       R.exit()
-      if ( debugger.value ) debugger.msg("Invoke is okay")
-      out.writeInt(OK)
+      if ( debugger.value ) debugger.msg("Function invocation is okay.")
+      socket.putScalarInt(OK)
     } catch {
       case e: Throwable =>
         R.exit()
-        out.writeInt(ERROR)
+        socket.putScalarInt(ERROR)
         e.printStackTrace(pw)
         pw.println(e + ( if ( e.getCause != null ) System.lineSeparator + e.getCause else "" ) )
     }
   }
 
-  private val RegexpVar = """^\s+(?:final )?\s*(?:override )?\s*var\s+([^(:]+):\s+([^=;]+).*""".r
-  private val RegexpVal = """^\s+(?:final )?\s*(?:override )?\s*val\s+([^(:]+):\s+([^=;]+).*""".r
-  private val RegexpDefNoParen = """^\s+(?:final )?\s*(?:override)?\s*def\s+([a-zA-Z0-9]+)(?:\[.*\]:|:)\s+([^=;]+).*""".r
-  private val RegexpDefWithParen = """^\s+(?:final )?\s*(?:override)?\s*def\s+([a-zA-Z0-9]+)(?:\[.*\]+)?\((.*)\):\s+([^=]+).*""".r
-  private val RegexpConstructor = """^\s+def\s+this\((.*)\).*""".r
-  private val RegexpThrows = """^\s*throws\s+.*""".r
-  private val rscalaReference = "rscalaReference"
-
-  private def parseArgNames(args: String): Array[String] = {
-    if ( args.contains(")(") ) null
-    else {
-      val x = args.split(":")
-      val y = if ( x.length > 1 ) x.dropRight(1) else x
-      y.map(z => {
-        val i = z.lastIndexOf(",")
-        if ( i >= 0 ) z.substring(i+1).trim else z.trim
-      })
-    }
-  }
-
   private def doScalap(): Unit = {
     if ( debugger.value ) debugger.msg("Doing scalap...")
-    val itemName = readString()
+    val itemName = socket.getScalarString()
     if ( debugger.value ) debugger.msg("... on "+itemName)
     val classpath = sys.props("sun.boot.class.path") + sys.props("path.separator") + sys.props("rscala.classpath")
     if ( debugger.value ) debugger.msg("... with classpath "+classpath)
     scala.tools.scalap.Main.main(Array("-cp",classpath,itemName))
-    out.writeInt(OK)
+    socket.putScalarInt(OK)
   }
 
   private def doSet(): Unit = {
-    val identifier = readString()
-    in.readInt() match {
+    val identifier = socket.getScalarString()
+    val shape = socket.getScalarInt()
+    shape match {
       case NULLTYPE =>
         if ( debugger.value ) debugger.msg("Setting null.")
         repl.bind(identifier,"Any",null)
       case REFERENCE =>
         if ( debugger.value ) debugger.msg("Setting reference.")
-        val originalIdentifier = readString()
+        val originalIdentifier = socket.getScalarString()
         try {
           val (value,tipe) = originalIdentifier match {
-            case cacheMap(value,typeOfTerm) => (value,typeOfTerm)
+            case cacheMap(v,typeOfTerm) => (v,typeOfTerm)
             case _ => (repl.valueOfTerm(originalIdentifier).get,typeOfTerm(originalIdentifier))
           }
           if ( repl.bind(identifier,tipe,value) != Success ) throw new RuntimeException("Cannot set reference.")
-          out.writeInt(OK)
+          socket.putScalarInt(OK)
         } catch {
           case e: Throwable =>
             if ( debugger.value ) debugger.msg("Caught exception: "+e)
-            out.writeInt(ERROR)
+            socket.putScalarInt(ERROR)
             e.printStackTrace(pw)
             pw.println(e + ( if ( e.getCause != null ) System.lineSeparator + e.getCause else "" ) )
         }
       case SCALAR =>
         if ( debugger.value ) debugger.msg("Setting atomic.")
-        val (v: Any, t: String) = in.readInt() match {
-          case INTEGER => (in.readInt(),"Int")
-          case DOUBLE => (in.readDouble(),"Double")
-          case BOOLEAN => (( in.readInt() != 0 ),"Boolean")
-          case STRING => (readString(),"String")
-          case BYTE => (in.readByte(),"Byte")
+        val tipe = socket.getScalarInt()
+        val (v: Any, t: String) = tipe match {
+          case INTEGER => (socket.getScalarInt(),    "Int")
+          case DOUBLE =>  (socket.getScalarDouble(), "Double")
+          case BOOLEAN => (socket.getScalarBoolean(),"Boolean")
+          case STRING =>  (socket.getScalarString(), "String")
+          case BYTE =>    (socket.getScalarByte(),   "Byte")
           case _ => throw new RuntimeException("Protocol error")
         }
         setAVM(identifier,t,v)
       case VECTOR =>
         if ( debugger.value ) debugger.msg("Setting vector...")
-        val length = in.readInt()
+        val (length, tipe) = socket.getTuple2Int()
         if ( debugger.value ) debugger.msg("... of length: "+length)
-        val (v, t): (Any,String) = in.readInt() match {
-          case INTEGER => (Array.fill(length) { in.readInt() },"Array[Int]")
-          case DOUBLE => (Array.fill(length) { in.readDouble() },"Array[Double]")
-          case BOOLEAN => (Array.fill(length) { ( in.readInt() != 0 ) },"Array[Boolean]")
-          case STRING => (Array.fill(length) { readString() },"Array[String]")
-          case BYTE => (Array.fill(length) { in.readByte() },"Array[Byte]")
+        val (v, t): (Any,String) = tipe match {
+          case INTEGER => (socket.getVectorInt(length),    "Array[Int]")
+          case DOUBLE =>  (socket.getVectorDouble(length), "Array[Double]")
+          case BOOLEAN => (socket.getVectorBoolean(length),"Array[Boolean]")
+          case STRING =>  (socket.getVectorString(length), "Array[String]")
+          case BYTE =>    (socket.getVectorByte(length),   "Array[Byte]")
           case _ => throw new RuntimeException("Protocol error")
         }
         setAVM(identifier,t,v)
       case MATRIX =>
         if ( debugger.value ) debugger.msg("Setting matrix...")
-        val nrow = in.readInt()
-        val ncol = in.readInt()
+        val (nrow, ncol, tipe) = socket.getTuple3Int()
         if ( debugger.value ) debugger.msg("... of dimensions: "+nrow+","+ncol)
-        val (v, t): (Any,String) = in.readInt() match {
-          case INTEGER => ( transposeIf( Array.fill(nrow) { Array.fill(ncol) { in.readInt() } }, rowMajor),"Array[Array[Int]]")
-          case DOUBLE => ( transposeIf( Array.fill(nrow) { Array.fill(ncol) { in.readDouble() } }, rowMajor),"Array[Array[Double]]")
-          case BOOLEAN => ( transposeIf( Array.fill(nrow) { Array.fill(ncol) { ( in.readInt() != 0 ) } }, rowMajor),"Array[Array[Boolean]]")
-          case STRING => ( transposeIf( Array.fill(nrow) { Array.fill(ncol) { readString() } }, rowMajor),"Array[Array[String]]")
-          case BYTE => ( transposeIf( Array.fill(nrow) { Array.fill(ncol) { in.readByte() } }, rowMajor),"Array[Array[Byte]]")
+        val (v, t): (Any,String) = tipe match {
+          case INTEGER => (socket.getMatrixInt(nrow,ncol,rowMajor),    "Array[Array[Int]]")
+          case DOUBLE =>  (socket.getMatrixDouble(nrow,ncol,rowMajor), "Array[Array[Double]]")
+          case BOOLEAN => (socket.getMatrixBoolean(nrow,ncol,rowMajor),"Array[Array[Boolean]]")
+          case STRING =>  (socket.getMatrixString(nrow,ncol,rowMajor), "Array[Array[String]]")
+          case BYTE =>    (socket.getMatrixByte(nrow,ncol,rowMajor),   "Array[Array[Byte]]")
           case _ => throw new RuntimeException("Protocol error")
         }
         setAVM(identifier,t,v)
@@ -234,7 +199,7 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
   }
 
   private def doGet(): Unit = {
-    val identifier = readString()
+    val identifier = socket.getScalarString()
     if ( debugger.value ) debugger.msg("Trying to get value of: "+identifier)
     val optionWithType = try {
       identifier match {
@@ -253,7 +218,7 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
     } catch {
       case e: Throwable =>
         if ( debugger.value ) debugger.msg("Caught exception: "+e)
-        out.writeInt(ERROR)
+        socket.putScalarInt(ERROR)
         e.printStackTrace(pw)
         pw.println(e + ( if ( e.getCause != null ) System.lineSeparator + e.getCause else "" ) )
         return
@@ -262,18 +227,18 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
     if ( optionWithType._1.isEmpty ) {
       if ( optionWithType._2 == "<notype>" ) {
         if ( debugger.value ) debugger.msg("... which does not exist.")
-        out.writeInt(UNDEFINED_IDENTIFIER)
+        socket.putScalarInt(UNDEFINED_IDENTIFIER)
         return
       } else {
-        if ( debugger.value ) debugger.msg("... which is emtpy [due to quirk in 2.10 series].")
-        out.writeInt(NULLTYPE)
+        if ( debugger.value ) debugger.msg("... which is empty [due to quirk in 2.10 series].")
+        socket.putScalarInt(NULLTYPE)
         return
       }
     }
     val v = optionWithType._1.get
     if ( v == null || v.isInstanceOf[Unit] ) {
       if ( debugger.value ) debugger.msg("... which is null")
-      out.writeInt(NULLTYPE)
+      socket.putScalarInt(NULLTYPE)
       return
     }
     val t = if ( optionWithType._2 == "Any" ) {
@@ -309,157 +274,97 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
     t match {
       case "Array[Int]" =>
         val vv = v.asInstanceOf[Array[Int]]
-        out.writeInt(VECTOR)
-        out.writeInt(vv.length)
-        out.writeInt(INTEGER)
-        for ( i <- 0 until vv.length ) out.writeInt(vv(i))
+        socket.putTuple3Int(VECTOR,vv.length,INTEGER)
+        socket.putVectorInt(vv)
       case "Array[Double]" =>
         val vv = v.asInstanceOf[Array[Double]]
-        out.writeInt(VECTOR)
-        out.writeInt(vv.length)
-        out.writeInt(DOUBLE)
-        for ( i <- 0 until vv.length ) out.writeDouble(vv(i))
+        socket.putTuple3Int(VECTOR,vv.length,DOUBLE)
+        socket.putVectorDouble(vv)
       case "Array[Boolean]" =>
         val vv = v.asInstanceOf[Array[Boolean]]
-        out.writeInt(VECTOR)
-        out.writeInt(vv.length)
-        out.writeInt(BOOLEAN)
-        for ( i <- 0 until vv.length ) out.writeInt(if ( vv(i) ) 1 else 0)
+        socket.putTuple3Int(VECTOR,vv.length,BOOLEAN)
+        socket.putVectorBoolean(vv)
       case "Array[String]" =>
         val vv = v.asInstanceOf[Array[String]]
-        out.writeInt(VECTOR)
-        out.writeInt(vv.length)
-        out.writeInt(STRING)
-        for ( i <- 0 until vv.length ) writeString(vv(i))
+        socket.putTuple3Int(VECTOR,vv.length,STRING)
+        socket.putVectorString(vv)
       case "Array[Byte]" =>
         val vv = v.asInstanceOf[Array[Byte]]
-        out.writeInt(VECTOR)
-        out.writeInt(vv.length)
-        out.writeInt(BYTE)
-        for ( i <- 0 until vv.length ) out.writeByte(vv(i))
+        socket.putTuple3Int(VECTOR,vv.length,BYTE)
+        socket.putVectorByte(vv)
       case "Array[Array[Int]]" =>
-        val vv1 = v.asInstanceOf[Array[Array[Int]]]
-        if ( isMatrix(vv1) ) {
-          val vv = transposeIf(vv1, rowMajor)
-          out.writeInt(MATRIX)
-          out.writeInt(vv.length)
-          if ( vv.length > 0 ) out.writeInt(vv(0).length)
-          else out.writeInt(0)
-          out.writeInt(INTEGER)
-          for ( i <- 0 until vv.length ) {
-            val vvv = vv(i)
-            for ( j <- 0 until vvv.length ) {
-              out.writeInt(vvv(j))
-            }
-          }
-        } else out.writeInt(UNSUPPORTED_STRUCTURE)
+        val vv = v.asInstanceOf[Array[Array[Int]]]
+        if ( socket.isMatrix(vv) ) {
+          val (nrow,ncol) = socket.rowsColumns(vv,rowMajor)
+          socket.putTuple4Int(MATRIX,nrow,ncol,INTEGER)
+          socket.putMatrixInt(vv,rowMajor)
+        } else socket.putScalarInt(UNSUPPORTED_STRUCTURE)
       case "Array[Array[Double]]" =>
-        val vv1 = v.asInstanceOf[Array[Array[Double]]]
-        if ( isMatrix(vv1) ) {
-          val vv = transposeIf(vv1, rowMajor)
-          out.writeInt(MATRIX)
-          out.writeInt(vv.length)
-          if ( vv.length > 0 ) out.writeInt(vv(0).length)
-          else out.writeInt(0)
-          out.writeInt(DOUBLE)
-          for ( i <- 0 until vv.length ) {
-            val vvv = vv(i)
-            for ( j <- 0 until vvv.length ) {
-              out.writeDouble(vvv(j))
-            }
-          }
-        } else out.writeInt(UNSUPPORTED_STRUCTURE)
+        val vv = v.asInstanceOf[Array[Array[Double]]]
+        if ( socket.isMatrix(vv) ) {
+          val (nrow,ncol) = socket.rowsColumns(vv,rowMajor)
+          socket.putTuple4Int(MATRIX,nrow,ncol,DOUBLE)
+          socket.putMatrixDouble(vv,rowMajor)
+        } else socket.putScalarInt(UNSUPPORTED_STRUCTURE)
       case "Array[Array[Boolean]]" =>
-        val vv1 = v.asInstanceOf[Array[Array[Boolean]]]
-        if ( isMatrix(vv1) ) {
-          val vv = transposeIf(vv1, rowMajor)
-          out.writeInt(MATRIX)
-          out.writeInt(vv.length)
-          if ( vv.length > 0 ) out.writeInt(vv(0).length)
-          else out.writeInt(0)
-          out.writeInt(BOOLEAN)
-          for ( i <- 0 until vv.length ) {
-            val vvv = vv(i)
-            for ( j <- 0 until vv(i).length ) {
-              out.writeInt(if ( vvv(j) ) 1 else 0)
-            }
-          }
-        } else out.writeInt(UNSUPPORTED_STRUCTURE)
+        val vv = v.asInstanceOf[Array[Array[Boolean]]]
+        if ( socket.isMatrix(vv) ) {
+          val (nrow,ncol) = socket.rowsColumns(vv,rowMajor)
+          socket.putTuple4Int(MATRIX,nrow,ncol,BOOLEAN)
+          socket.putMatrixBoolean(vv,rowMajor)
+        } else socket.putScalarInt(UNSUPPORTED_STRUCTURE)
       case "Array[Array[String]]" =>
-        val vv1 = v.asInstanceOf[Array[Array[String]]]
-        if ( isMatrix(vv1) ) {
-          val vv = transposeIf(vv1, rowMajor)
-          out.writeInt(MATRIX)
-          out.writeInt(vv.length)
-          if ( vv.length > 0 ) out.writeInt(vv(0).length)
-          else out.writeInt(0)
-          out.writeInt(STRING)
-          for ( i <- 0 until vv.length ) {
-            val vvv = vv(i)
-            for ( j <- 0 until vv(i).length ) {
-              writeString(vvv(j))
-            }
-          }
-        } else out.writeInt(UNSUPPORTED_STRUCTURE)
+        val vv = v.asInstanceOf[Array[Array[String]]]
+        if ( socket.isMatrix(vv) ) {
+          val (nrow,ncol) = socket.rowsColumns(vv,rowMajor)
+          socket.putTuple4Int(MATRIX,nrow,ncol,STRING)
+          socket.putMatrixString(vv,rowMajor)
+        } else socket.putScalarInt(UNSUPPORTED_STRUCTURE)
       case "Array[Array[Byte]]" =>
-        val vv1 = v.asInstanceOf[Array[Array[Byte]]]
-        if ( isMatrix(vv1) ) {
-          val vv = transposeIf(vv1, rowMajor)
-          out.writeInt(MATRIX)
-          out.writeInt(vv.length)
-          if ( vv.length > 0 ) out.writeInt(vv(0).length)
-          else out.writeInt(0)
-          out.writeInt(BYTE)
-          for ( i <- 0 until vv.length ) {
-            val vvv = vv(i)
-            for ( j <- 0 until vvv.length ) {
-              out.writeByte(vvv(j))
-            }
-          }
-        } else out.writeInt(UNSUPPORTED_STRUCTURE)
+        val vv = v.asInstanceOf[Array[Array[Byte]]]
+        if ( socket.isMatrix(vv) ) {
+          val (nrow,ncol) = socket.rowsColumns(vv,rowMajor)
+          socket.putTuple4Int(MATRIX,nrow,ncol,BYTE)
+          socket.putMatrixByte(vv,rowMajor)
+        } else socket.putScalarInt(UNSUPPORTED_STRUCTURE)
       case "Int" =>
-        out.writeInt(SCALAR)
-        out.writeInt(INTEGER)
-        out.writeInt(v.asInstanceOf[Int])
+        socket.putTuple2Int(SCALAR,INTEGER)
+        socket.putScalarInt(v.asInstanceOf[Int])
       case "Double" =>
-        out.writeInt(SCALAR)
-        out.writeInt(DOUBLE)
-        out.writeDouble(v.asInstanceOf[Double])
+        socket.putTuple2Int(SCALAR,DOUBLE)
+        socket.putScalarDouble(v.asInstanceOf[Double])
       case "Boolean" =>
-        out.writeInt(SCALAR)
-        out.writeInt(BOOLEAN)
-        out.writeInt(if (v.asInstanceOf[Boolean]) 1 else 0)
+        socket.putTuple2Int(SCALAR,BOOLEAN)
+        socket.putScalarBoolean(v.asInstanceOf[Boolean])
       case "String" =>
-        out.writeInt(SCALAR)
-        out.writeInt(STRING)
-        writeString(v.asInstanceOf[String])
+        socket.putTuple2Int(SCALAR,STRING)
+        socket.putScalarString(v.asInstanceOf[String])
       case "Byte" =>
-        out.writeInt(SCALAR)
-        out.writeInt(BYTE)
-        out.writeByte(v.asInstanceOf[Byte])
+        socket.putTuple2Int(SCALAR,BYTE)
+        socket.putScalarByte(v.asInstanceOf[Byte])
       case _ =>
         if ( debugger.value ) debugger.msg("Bowing out: "+identifier)
-        out.writeInt(UNSUPPORTED_STRUCTURE)
+        socket.putScalarInt(UNSUPPORTED_STRUCTURE)
     }
   }
 
   private def doGetReference(): Unit = {
-    val identifier = readString()
+    val identifier = socket.getScalarString()
     if ( debugger.value ) debugger.msg("Trying to get reference for: "+identifier)
     identifier match {
       case "?" =>
-        out.writeInt(OK)
-        writeString(cacheMap.store(functionResult))
-        writeString(functionResult._2)
+        socket.putScalarInt(OK)
+        socket.putScalarString(cacheMap.store(functionResult))
+        socket.putScalarString(functionResult._2)
       case cacheMap(value, typeOfTerm) =>
-        out.writeInt(OK)
-        writeString(identifier)
-        writeString(typeOfTerm)
+        socket.putScalarInt(OK)
+        socket.putScalarString(identifier)
+        socket.putScalarString(typeOfTerm)
       case _ =>
         if ( identifier == "null" ) {
-          out.writeInt(OK)
-          writeString("null")
-          writeString("Null")
+          socket.putScalarInt(OK)
+          socket.putScalarString("null")
+          socket.putScalarString("Null")
           return
         }
         val (id, opt) = try {
@@ -469,16 +374,16 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
         } catch {
           case e: Throwable =>
             if ( debugger.value ) debugger.msg("Caught exception: "+e)
-            out.writeInt(ERROR)
+            socket.putScalarInt(ERROR)
             e.printStackTrace(pw)
             pw.println(e + ( if ( e.getCause != null ) System.lineSeparator + e.getCause else "" ) )
             return
         }
-        if ( opt.isEmpty ) out.writeInt(UNDEFINED_IDENTIFIER)
+        if ( opt.isEmpty ) socket.putScalarInt(UNDEFINED_IDENTIFIER)
         else {
-          out.writeInt(OK)
-          writeString(id)
-          writeString(typeOfTerm(id))
+          socket.putScalarInt(OK)
+          socket.putScalarString(id)
+          socket.putScalarString(typeOfTerm(id))
         }
     }
   }
@@ -488,10 +393,7 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
     request match {
       case SHUTDOWN =>
         if ( debugger.value ) debugger.msg("Shutting down")
-        in.close()
-        out.close()
-        socketIn.close()
-        socketOut.close()
+        socket.close()
         return false
       case EXIT =>
         if ( debugger.value ) debugger.msg("Exiting")
@@ -520,7 +422,7 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
     while ( true ) {
       if ( debugger.value ) debugger.msg("Scala server at top of the loop waiting for a command.")
       val request = try {
-        in.readInt()
+        socket.getScalarInt()
       } catch {
         case _: Throwable =>
           return
@@ -532,13 +434,13 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
           }
         }
         pw.flush()
-        writeString(baosOut.toString+baosErr.toString)
+        socket.putScalarString(baosOut.toString+baosErr.toString)
         baosOut.reset()
         baosErr.reset()
       } else {
         if ( ! heart(request) ) return
       }
-      out.flush()
+      socket.flush()
     }
   }
 
@@ -546,7 +448,7 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
 
 object ScalaServer {
 
-  def apply(portsFilename: String, debug: Boolean = false, serializeOutput: Boolean = false, rowMajor: Boolean = true, port: Int = 0): ScalaServer = {
+  def apply(portFilename: String, debug: Boolean = false, serializeOutput: Boolean = false, rowMajor: Boolean = true, port: Int = 0): ScalaServer = {
     // Set classpath
     val settings = new Settings()
     settings.embeddedDefaults[RClient]
@@ -568,7 +470,7 @@ object ScalaServer {
     }
     m1.setAccessible(true)
     m1.invoke(settings.language,"reflectiveCalls")
-    // A better way to do it, but Scala 2.10.x doess not the settings.language.add method.
+    // A better way to do it, but Scala 2.10.x does not the settings.language.add method.
     // settings.language.add("reflectiveCalls")
     // Set up sinks
     val (debugger,prntWrtr,baosOut,baosErr) = serializeOutput match {
@@ -595,7 +497,7 @@ object ScalaServer {
     // iloop.intp = intp
     // iloop.verbosity()
     // Return the server
-    new ScalaServer(intp, prntWrtr, baosOut, baosErr, portsFilename, debugger, serializeOutput, rowMajor, port)
+    new ScalaServer(intp, prntWrtr, baosOut, baosErr, portFilename, debugger, serializeOutput, rowMajor, port, 1*1024*1024)
   }
 
 }
